
--[[
-- Simple wrapper over gh_imgui lib for frame/window + begin/end functions.
--]]


_imgui_initialized = 0


function imgui_frame_begin()

  if (_imgui_initialized == 0) then
    gh_imgui.init()
    _imgui_initialized = 1
  end

  local win_w, win_h = gh_window.getsize(0)

  local LEFT_BUTTON = 1
  local mouse_left_button = gh_input.mouse_get_button_state(LEFT_BUTTON) 
  local RIGHT_BUTTON = 2
  local mouse_right_button = gh_input.mouse_get_button_state(RIGHT_BUTTON) 
  local mouse_x, mouse_y = gh_input.mouse_get_position()

  gh_imgui.frame_begin(win_w, win_h, mouse_x, mouse_y, mouse_left_button, mouse_right_button)
end  


function imgui_frame_end()

  gh_imgui.frame_end()

end


function imgui_window_begin_v1(label, width, height, posx, posy)

  -- Flags for window style, window position and window size.
  --
  local window_default = 0
  local window_no_resize = 2
  local window_no_move = 4
  local window_no_collapse = 32
  local window_show_border = 128
  local window_no_save_settings = 256
  local pos_size_flag_always = 1 -- Always set the pos and/or size
  local pos_size_flag_once = 2 -- Set the pos and/or size once per runtime session (only the first call with succeed)
  local pos_size_flag_first_use_ever = 4  -- Set the pos and/or size if the window has no saved data (if doesn't exist in the .ini file)
  local pos_size_flag_appearing = 8  -- Set the pos and/or size if the window is appearing after being hidden/inactive (or the first time)

  -- Beginning of the window with caption "Gear control"
  --  

  local window_flags = window_default

  local is_open = gh_imgui.window_begin(label, width, height, posx, posy, window_flags, pos_size_flag_first_use_ever, pos_size_flag_first_use_ever)
  return is_open
end



function imgui_window_end()

  gh_imgui.window_end()

end


function imgui_vertical_space()
  local IMGUI_WIDGET_VERTICAL_SPACING = 4
  gh_imgui.widget(IMGUI_WIDGET_VERTICAL_SPACING)
end  

function imgui_separator()
  local IMGUI_WIDGET_SEPARATOR = 1
  gh_imgui.widget(IMGUI_WIDGET_SEPARATOR)
end  

function imgui_same_line()
  local IMGUI_WIDGET_SAME_LINE = 2
  gh_imgui.widget(IMGUI_WIDGET_SAME_LINE)
end  

function imgui_bullet()
  local IMGUI_WIDGET_BULLET = 3
  gh_imgui.widget(IMGUI_WIDGET_BULLET)
end  

